@@ -85,9 +85,9 @@ test with special configuration once, without affecting other code:
 
 =cut
 
-our $VERSION = '0.001003';
+our $VERSION = '0.002';
 our @EXPORT = qw/analyse_code star_rating_of_warnings/;
-our @EXPORT_OK = (@EXPORT, qw/test_lines test_clang_tidy/);
+our @EXPORT_OK = (@EXPORT, qw/test_lines test_clang_tidy test_lizard/);
 our %EXPORT_TAGS = (default => \@EXPORT, all => \@EXPORT_OK);
 
 # set this to a "Test::More::diag"-like function to get debug output
@@ -96,7 +96,9 @@ our $DEBUG;
 use Carp qw/carp croak/;
 use Cpanel::JSON::XS qw/encode_json/;
 use File::Temp qw//;
+use File::Which qw/which/;
 use List::Util qw/reduce any/;
+use Text::CSV qw/csv/;
 
 sub diag { $DEBUG->(@_) if defined $DEBUG }
 
@@ -185,16 +187,20 @@ each of which is a list of globs such as C<modernize-*>. The checks in
 C<@clang_tidy_warnings> produce warnings, while the checks in
 C<@clang_tidy_errors> produce errors. There is also a hash
 C<%clang_tidy_check_options> which contains configuration for the
-checks.
+checks. Finally, the path to the clang-tidy executable is
+C<$clang_tidy_path>, which is initialized by looking in the PATH using
+L<File::Which>. Set this variable to undef to disable this test.
 
 This test does not require a reference, but is limited to languages
 that clang-tidy understands. This is controlled by the global variable
-C<%extension_of_language>, which contains file extensions for the
-supported languages.
+C<%clang_tidy_extension_of_language>, which contains file extensions
+for the supported languages.
 
 =cut
 
-our %extension_of_language = (
+our $clang_tidy_path = which 'clang-tidy';
+
+our %clang_tidy_extension_of_language = (
 	'C' => '.c',
 	'C++' => '.cpp',
 );
@@ -219,25 +225,25 @@ our %clang_tidy_check_options = (
 	'readability-implicit-bool-conversion.AllowIntegerConditions' => 1,
 );
 
-sub _clang_tidy_exists {
-	# does clang-tidy exist?
-	# run it with no arguments, see if exit code is 127
-	system 'clang-tidy 2>/dev/null 1>/dev/null';
-	$? >> 8 != 127
-}
-
-sub test_clang_tidy {
-	my %args = @_;
-	my $extension = $extension_of_language{uc $args{language}};
-	return unless defined $extension;
-
+sub write_code_to_file {
+	my ($code, $extension) = @_;
 	my $fh = File::Temp->new(
 		TEMPLATE => 'code-qualityXXXXX',
 		TMPDIR => 1,
 		SUFFIX => $extension,
 	);
-	print $fh $args{code} or croak 'Failed to write code to temporary file';
+	print $fh $code or croak 'Failed to write code to temporary file';
 	close $fh or croak 'Failed to close temporary file';
+	$fh
+}
+
+sub test_clang_tidy {
+	my %args = @_;
+	return unless $clang_tidy_path;
+	my $extension = $clang_tidy_extension_of_language{uc $args{language}};
+	return unless defined $extension;
+
+	my $fh = write_code_to_file $args{code}, $extension;
 
 	my $checks = join ',', '-*', @clang_tidy_warnings, @clang_tidy_errors;
 	my $errors = join ',', '-*', @clang_tidy_errors;
@@ -251,13 +257,10 @@ sub test_clang_tidy {
 		WarningsAsErrors => $errors,
 	};
 
-	my @output = qx,clang-tidy -config='$config' $fh 2>/dev/null,;
-	my $exit_code = $? >> 8; # this is usually the number of clang-tidy errors
+	my @output = qx,$clang_tidy_path -config='$config' $fh 2>/dev/null,;
 	my $signal = $? & 127;
-	if ($signal || ($exit_code == 127 && !_clang_tidy_exists)) {
-		# special case: exit code 127 could mean "127 errors found" or
-		# "clang-tidy not found"
-		carp "Failed to run clang-tidy, \$? is $?";
+	if ($signal) {
+		carp "Failed to run $clang_tidy_path, \$? is $?";
 		return
 	}
 
@@ -274,6 +277,80 @@ sub test_clang_tidy {
 	\@warnings
 }
 
+=head3 test_lizard
+
+This test runs the L<lizard.py|https://github.com/terryyin/lizard>
+code complexity analyser on the code, and reports a warning for every
+function that has high cyclomatic complexity, or that is too long.
+
+The thresholds that determine whether a warning or an error are raised
+are determined by four global variables, C<$lizard_warning_loc>,
+C<$lizard_error_loc>, C<$lizard_warning_ccn>, C<$lizard_error_ccn>.
+Finally, the path to the lizard executable is C<$lizard_path>, which
+is initialized by looking in the PATH using L<File::Which>. Set this
+variable to undef to disable this test.
+
+This test does not require a reference, but is limited to languages
+that lizard understands. This is controlled by the global variable
+C<%lizard_extension_of_language>, which contains file extensions for
+the supported languages.
+
+=cut
+
+our $lizard_path = which 'lizard.py' // which 'lizard';
+
+our %lizard_extension_of_language = (
+	'C' => '.c',
+	'C++' => '.cpp',
+	'JAVA' => '.java',
+);
+
+our $lizard_warning_loc = 70;
+our $lizard_error_loc = 140;
+our $lizard_warning_ccn = 15;
+our $lizard_error_ccn = 25;
+
+# call each of these functions with line number as argument
+sub warn_function_is_long { [warning => "This function is over $lizard_warning_loc lines long [long-function]", shift, 0] }
+sub warn_function_is_very_long { [error => "This function is over $lizard_error_loc lines long [very-long-function]", shift, 0] }
+sub warn_function_is_complex { [warning => "Cyclomatic complexity of this function is over $lizard_warning_ccn \[high-complexity]", shift, 0] }
+sub warn_function_is_very_complex { [error => "Cyclomatic complexity of this function is over $lizard_error_ccn \[very-high-complexity]", shift, 0] }
+
+sub test_lizard {
+	my %args = @_;
+	return unless $lizard_path;
+	my $extension = $lizard_extension_of_language{uc $args{language}};
+	return unless defined $extension;
+
+	my $fh = write_code_to_file $args{code}, $extension;
+
+	my $output = qx,$lizard_path --csv -i -1 -m -V $fh 2>/dev/null,;
+	if ($?) { # we expect lizard to always exit 0, unless there is a problem
+		carp "Failed to run $lizard_path, \$? is $?";
+		return
+	}
+
+	my $csv = csv in => \$output, headers => 'auto';
+	unless ($csv) {
+		carp 'Failed to parse output of lizard.py as CSV';
+		return
+	}
+
+	my @warnings;
+	for my $line (@$csv) {
+		my %line = %$line;
+		diag "$line{function} has $line{NLOC} loc and $line{CCN} ccn";
+		push @warnings,
+			$line{NLOC} > $lizard_error_loc ? warn_function_is_very_long $line{start} :
+			$line{NLOC} > $lizard_warning_loc ? warn_function_is_long $line{start} : ();
+		push @warnings,
+			$line{CCN} > $lizard_error_ccn ? warn_function_is_very_complex $line{start} :
+			$line{CCN} > $lizard_warning_ccn ? warn_function_is_complex $line{start} : ();
+	}
+
+	\@warnings
+}
+
 =head3 analyse_code
 
 B<analyse_code> runs every test above on the code, producing a
@@ -286,6 +363,7 @@ C<@all_tests>, which is a list of coderefs.
 our @all_tests = (
 	\&test_lines,
 	\&test_clang_tidy,
+	\&test_lizard,
 );
 
 sub analyse_code {
